using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.SignalR;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Net.Http.Headers;
using NetworkResurrector.Api.Application.Services.Abstractions;
using NetworkResurrector.Api.Services;
using System;
using System.Security.Claims;

namespace NetworkResurrector.Api.Extensions
{
    public static class SignalRExtensions
    {
        private const string AUTH_QUERY_STRING_KEY = "access_token";

        /// <summary>
        /// Middleware that extracts a bearer token from the query string (using the "access_token" key)
        /// and injects it into the Authorization header. This is necessary for SignalR connections,
        /// which cannot send authentication tokens via headers during the WebSocket handshake.
        /// Enables standard authentication handlers to process SignalR requests.
        /// </summary>
        public static void UseSignalRAuthentication(this IApplicationBuilder app)
        {
            app.Use(async (context, next) =>
            {
                var headers = context.Request.Headers;
                if (string.IsNullOrWhiteSpace(headers[HeaderNames.Authorization]) &&
                    context.Request.Query.TryGetValue(AUTH_QUERY_STRING_KEY, out var token) &&
                    !string.IsNullOrWhiteSpace(token))
                {
                    try
                    {
                        // Overwrite or set the Authorization header with the Bearer token from the query string
                        headers[HeaderNames.Authorization] = $"Tuitio {token}";
                    }
                    catch (InvalidOperationException)
                    {
                        // Ignore if setting the header fails (e.g., if headers are read-only at this point)
                    }
                }

                await next.Invoke();
            });
        }

        /// <summary>
        /// Gets the user identifier for the current connection
        /// </summary>
        /// <param name="context">HubCallerContext</param>
        /// <returns>The user id or null if not authenticated</returns>
        public static string GetUserId(this HubCallerContext context)
        {
            return context.User?.FindFirst(ClaimTypes.NameIdentifier)?.Value;
        }

        /// <summary>
        /// Gets the username for the current connection
        /// </summary>
        /// <param name="context">HubCallerContext</param>
        /// <returns>The username or null if not authenticated</returns>
        public static string GetUsername(this HubCallerContext context)
        {
            return context.User?.FindFirst(ClaimTypes.Name)?.Value;
        }

        public static void AddSignalRNotifications(this IServiceCollection services)
        {
            services.AddSignalR();
            services.AddScoped<IMessageHubPublisher, MessageHubPublisher>();
        }
    }
}
