using System;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.SignalR;
using Microsoft.Extensions.Logging;
using NetworkResurrector.Api.Extensions;

namespace NetworkResurrector.Api.Hubs
{
    [Authorize]
    public class MessageHub : Hub
    {
        private readonly ILogger<MessageHub> _logger;

        public MessageHub(ILogger<MessageHub> logger)
        {
            _logger = logger;
        }        
        
        public override async Task OnConnectedAsync()
        {
            var userId = Context.GetUserId();
            var username = Context.GetUsername();
            
            _logger.LogInformation($"Client connected: ConnectionId={Context.ConnectionId}, UserId={userId}, Username={username}");

            if (!string.IsNullOrEmpty(username))
            {
                var groupName = GetGroupName(username);
                await Groups.AddToGroupAsync(Context.ConnectionId, groupName);
                _logger.LogInformation($"Added connection {Context.ConnectionId} to user group {groupName}");
            }

            await base.OnConnectedAsync();
        }

        public override async Task OnDisconnectedAsync(Exception exception)
        {
            var userId = Context.GetUserId();
            var username = Context.GetUsername();
            
            _logger.LogInformation($"Client disconnected: ConnectionId={Context.ConnectionId}, UserId={userId}, Username={username}");

            if (!string.IsNullOrEmpty(username))
            {
                var groupName = GetGroupName(username);
                await Groups.RemoveFromGroupAsync(Context.ConnectionId, groupName);
                _logger.LogInformation($"Removed connection {Context.ConnectionId} from user group {groupName}");
            }

            await base.OnDisconnectedAsync(exception);
        }

        private string GetGroupName(string id) => $"User_{id}";

        public async Task SendMessage(string message)
        {
            _logger.LogInformation($"Received message from {Context.ConnectionId}: {message}");
            await Clients.All.SendAsync("ReceiveMessage", message);
        }

        public async Task SendNotification(string title, string message)
        {
            _logger.LogInformation($"Sending notification: {title} - {message}");
            await Clients.All.SendAsync("ReceiveNotification", title, message);
        }

        public async Task JoinGroup(string groupName)
        {
            await Groups.AddToGroupAsync(Context.ConnectionId, groupName);
            _logger.LogInformation($"Client {Context.ConnectionId} joined group: {groupName}");
            await Clients.Group(groupName).SendAsync("GroupNotification", $"User joined {groupName} group");
        }
        
        public async Task SendToGroup(string groupName, string message)
        {
            _logger.LogInformation($"Message to group {groupName}: {message}");
            await Clients.Group(groupName).SendAsync("ReceiveGroupMessage", groupName, message);
        }

        public async Task SendToClient(string connectionId, string message)
        {
            _logger.LogInformation($"Sending message to client {connectionId}: {message}");
            await Clients.Client(connectionId).SendAsync("ReceiveMessage", message);
        }

        public async Task SendToUser(string userId, string title, string message)
        {
            var senderUserId = Context.GetUserId();
            var senderUsername = Context.GetUsername();
            
            _logger.LogInformation($"Sending notification from user {senderUserId} ({senderUsername}) to user {userId}: {title} - {message}");
            await Clients.User(userId).SendAsync("ReceiveNotification", title, message);
        }
    }
}
