﻿using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.SignalR;
using Microsoft.Extensions.Logging;
using NetworkResurrector.Api.Application.Services.Abstractions;
using NetworkResurrector.Api.Domain.Models.InternalNotifications;
using NetworkResurrector.Api.Hubs;
using System;
using System.Security.Claims;
using System.Threading;
using System.Threading.Tasks;

namespace NetworkResurrector.Api.Services
{
    public class MessageHubPublisher : IMessageHubPublisher
    {
        private const string ConnectionIdHeader = "SignalR-ConnectionId";

        private readonly IHubContext<MessageHub> _hubContext;
        private readonly ILogger<MessageHubPublisher> _logger;
        private readonly string _connectionId;
        private readonly string _userId;

        public MessageHubPublisher(IHubContext<MessageHub> hubContext, ILogger<MessageHubPublisher> logger, IHttpContextAccessor httpContextAccessor)
        {
            _hubContext = hubContext;
            _logger = logger;

            var headers = httpContextAccessor.HttpContext?.Request.Headers;
            if (headers.TryGetValue(ConnectionIdHeader, out var connectionIdValue) == true)
                _connectionId = connectionIdValue.ToString();
            else
                _connectionId = string.Empty;

            _userId = httpContextAccessor.HttpContext?.User?.FindFirst(ClaimTypes.NameIdentifier)?.Value;
        }

        /// <summary>
        /// Broadcasts a message to all connected clients.
        /// </summary>
        public async Task Broadcast<T>(T message, CancellationToken cancellationToken = default) where T : class, IRealtimeMessage
        {
            if (message is null)
                throw new ArgumentNullException(nameof(message), "Message cannot be null");

            var notification = RealtimeNotification<T>.Create(message, _connectionId);
            _logger.LogInformation($"Broadcasting message of type: {notification.Type}.");

            await _hubContext.Clients.All.SendAsync("ReceiveNotification", notification, cancellationToken);
        }

        /// <summary>
        /// Sends a message to the connected client identified by the connection ID.
        /// </summary>
        public async Task Send<T>(T message, CancellationToken cancellationToken = default) where T : class, IRealtimeMessage
        {
            if (message is null)
                throw new ArgumentNullException(nameof(message), "Message cannot be null");

            var notification = RealtimeNotification<T>.Create(message, _connectionId);
            _logger.LogInformation($"Sending message of type: {notification.Type}.");

            await _hubContext.Clients.Client(_connectionId).SendAsync("ReceiveNotification", notification, cancellationToken);
        }

        /// <summary>
        /// Sends a message to the authenticated user identified by the user ID.
        /// </summary>
        public async Task SendToUser<T>(T message, CancellationToken cancellationToken = default) where T : class, IRealtimeMessage
        {
            if (message is null)
                throw new ArgumentNullException(nameof(message), "Message cannot be null");

            if (string.IsNullOrEmpty(_userId))
                throw new InvalidOperationException("User ID is not available. Ensure the user is authenticated.");

            var notification = RealtimeNotification<T>.Create(message, _connectionId);
            _logger.LogInformation($"Sending message of type: {notification.Type}.");

            await _hubContext.Clients.User(_userId).SendAsync("ReceiveNotification", notification, cancellationToken);
        }
    }
}
