import React, { useCallback, useState } from "react";
import { Badge, IconButton, Popover } from "@mui/material";
import { Notifications } from "@mui/icons-material";
import PopoverContent from "./PopoverContent";
import { AppNotificationPayload, AppNotification, NotificationLevel, EmailSentPayload } from "./types";
import { shortid } from "utils/uid";
import { Hint } from "components/common";
import { RealtimeNotification } from "units/notifications/types";
import { notificationTypes } from "../../../constants";
import { useSubscription } from "hooks";

const UNREAD_NOTIFICATIONS_DOT_LIMIT = 9;

type Props = {
  disabled?: boolean;
};

const AppNotifications: React.FC<Props> = ({ disabled }) => {
  const [anchorEl, setAnchorEl] = useState<HTMLButtonElement | null>(null);
  const [notifications, setNotifications] = useState<AppNotification[]>([]);

  const push = useCallback((notification: AppNotification) => {
    setNotifications(prev => [...prev, notification]);
  }, []);

  const handleNotificationReceive = useCallback(
    (notification: RealtimeNotification<AppNotificationPayload>) => {
      const n: AppNotification = {
        ...notification.payload,
        id: shortid(),
        moment: notification.payload.moment ?? new Date(),
        level: notification.payload.level ?? NotificationLevel.INFO,
        read: false
      };
      push(n);
    },
    [push]
  );

  useSubscription<AppNotificationPayload>(notificationTypes.APP_NOTIFICATION_RECEIVED, {
    onNotification: handleNotificationReceive
  });

  useSubscription<EmailSentPayload>(notificationTypes.EMAIL_SENT, {
    onNotification: notification => {
      const n: AppNotification = {
        id: shortid(),
        moment: new Date(),
        level: NotificationLevel.INFO,
        content: `Email was sent to ${notification.payload.to} informing that the machine ${notification.payload.machineName} has been woken up.`,
        read: false
      };
      push(n);
    }
  });

  const handleClick = (event: React.MouseEvent<HTMLButtonElement>) => {
    setAnchorEl(event.currentTarget);
  };

  const handleRemove = useCallback((id: string) => {
    setNotifications(prev => prev.filter(notification => notification.id !== id));
  }, []);

  const handleMarkAllAsRead = useCallback(() => {
    setNotifications(prev => prev.map(notification => ({ ...notification, read: true })));
  }, []);

  const handleClose = () => {
    handleMarkAllAsRead();
    setAnchorEl(null);
  };

  const open = Boolean(anchorEl);
  const id = open ? "app-notifications-popover" : undefined;
  const unread = notifications.filter(notification => !notification.read).length;
  const variant = unread > UNREAD_NOTIFICATIONS_DOT_LIMIT ? "dot" : "standard";

  return (
    <>
      <Hint title="Notifications" arrow>
        <IconButton aria-label={id} color="inherit" disabled={disabled} onClick={handleClick}>
          <Badge
            color="warning"
            badgeContent={unread}
            variant={variant}
            overlap="circular"
            anchorOrigin={{
              vertical: "top",
              horizontal: "right"
            }}
          >
            <Notifications />
          </Badge>
        </IconButton>
      </Hint>
      <Popover
        id={id}
        open={open}
        anchorEl={anchorEl}
        onClose={handleClose}
        anchorOrigin={{
          vertical: "bottom",
          horizontal: "left"
        }}
        slotProps={{ paper: { sx: { width: 450 } } }}
      >
        <PopoverContent notifications={notifications} onRemove={handleRemove} onMarkAllAsRead={handleMarkAllAsRead} />
      </Popover>
    </>
  );
};

export default AppNotifications;
