import React, { useMemo, useState } from "react";
import { Avatar, Box, IconButton, List, ListItem, ListItemAvatar, ListItemText, Tooltip, Typography } from "@mui/material";
import { AppNotification, NotificationLevel } from "./types";
import {
  MarkEmailReadOutlined,
  CloseOutlined,
  InfoOutlined,
  CheckCircleOutlined,
  ReportOutlined,
  ErrorOutlineOutlined,
  MarkEmailUnreadOutlined,
  EmailOutlined
} from "@mui/icons-material";
import { styled } from "@mui/material/styles";

const PrimarySpan = styled("span")(({ theme }) => ({
  color: theme.palette.primary.main
}));

type Props = {
  notifications: AppNotification[];
  onRemove: (id: string) => void;
  onMarkAllAsRead: () => void;
};

const PopoverContent: React.FC<Props> = ({ notifications, onRemove, onMarkAllAsRead }) => {
  const [viewUnreadOnly, setViewUnreadOnly] = useState<boolean>(false);

  const isEmpty = notifications.length === 0;

  const filteredNotifications = useMemo(() => {
    const list = viewUnreadOnly ? notifications.filter(n => !n.read) : [...notifications];
    list.sort((a, b) => b.moment.getTime() - a.moment.getTime());
    return list;
  }, [viewUnreadOnly, notifications]);

  return (
    <>
      <Box
        sx={{
          alignItems: "center",
          backgroundColor: "primary.main",
          color: "primary.contrastText",
          display: "flex",
          justifyContent: "space-between",
          px: 1,
          py: 0.5
        }}
      >
        <Typography color="inherit" variant="subtitle1">
          Notifications
        </Typography>
        <Box
          sx={{
            display: "flex",
            justifyContent: "flex-end"
          }}
        >
          <Tooltip title={viewUnreadOnly ? "View all" : "View unread only"}>
            <span>
              <IconButton onClick={() => setViewUnreadOnly(prev => !prev)} size="small" color="inherit" disabled={isEmpty}>
                {viewUnreadOnly ? <EmailOutlined fontSize="small" /> : <MarkEmailUnreadOutlined fontSize="small" />}
              </IconButton>
            </span>
          </Tooltip>
          <Tooltip title="Mark all as read">
            <span>
              <IconButton onClick={onMarkAllAsRead} size="small" color="inherit" disabled={isEmpty}>
                <MarkEmailReadOutlined fontSize="small" />
              </IconButton>
            </span>
          </Tooltip>
        </Box>
      </Box>
      {isEmpty ? (
        <Box sx={{ p: 1 }}>
          <Typography variant="subtitle2" sx={{ opacity: 0.6 }}>
            There are no notifications
          </Typography>
        </Box>
      ) : (
        <Box
          sx={{
            maxHeight: 400,
            overflowY: "auto",
            "&::-webkit-scrollbar": {
              width: "8px"
            },
            "&::-webkit-scrollbar-thumb": {
              backgroundColor: "rgba(0,0,0,0.2)",
              borderRadius: "4px"
            },
            "&::-webkit-scrollbar-track": {
              backgroundColor: "rgba(0,0,0,0.05)"
            }
          }}
        >
          <List disablePadding>
            {filteredNotifications.map(notification => (
              <ListItem
                divider
                key={notification.id}
                sx={{
                  alignItems: "flex-start",
                  "&:hover": { backgroundColor: "action.hover" },
                  "& .MuiListItemSecondaryAction-root": { top: "24%" }
                }}
                secondaryAction={
                  <Tooltip title="Remove">
                    <IconButton edge="end" onClick={() => onRemove(notification.id)} size="small">
                      <CloseOutlined sx={{ fontSize: 14 }} />
                    </IconButton>
                  </Tooltip>
                }
              >
                {getNotificationContent(notification)}
              </ListItem>
            ))}
          </List>
        </Box>
      )}
    </>
  );
};

const getNotificationContent = (notification: AppNotification): React.ReactElement | null => {
  const isSuccess = notification.level === NotificationLevel.SUCCESS;
  const isWarning = notification.level === NotificationLevel.WARNING;
  const isError = notification.level === NotificationLevel.ERROR;

  const RowIcon = isSuccess ? (
    <CheckCircleOutlined color="success" />
  ) : isWarning ? (
    <ReportOutlined color="warning" />
  ) : isError ? (
    <ErrorOutlineOutlined color="error" />
  ) : (
    <InfoOutlined color="info" />
  );

  return (
    <>
      <ListItemAvatar sx={{ mt: 0.5 }}>
        <Avatar sx={{ bgcolor: "rgba(0, 0, 0, 0.1)", color: "primary.main" }}>{RowIcon}</Avatar>
      </ListItemAvatar>
      <ListItemText
        primary={
          <Box
            sx={{
              alignItems: "flex-start",
              display: "flex",
              flexDirection: "column"
            }}
          >
            <Typography sx={{ mb: 0.5, fontWeight: "bold" }} variant="body2">
              {notification.author && <PrimarySpan>{notification.author} </PrimarySpan>}
              {notification.title}
            </Typography>
            <Typography variant="body2">{notification.content}</Typography>
          </Box>
        }
        secondary={
          <Typography color="textSecondary" variant="caption">
            {notification.moment.toLocaleString()}
          </Typography>
        }
        sx={{ my: 0 }}
      />
    </>
  );
};

export default PopoverContent;
