import React, { useState } from "react";
import { Box, Button, Card, CardContent, Grid, TextField, Typography } from "@mui/material";
import SendIcon from "@mui/icons-material/Send";
import axios from "axios";
import { useRealtimeNotifications } from "units/notifications/hooks";
import env from "utils/env";
import { acquire as fetchTuitioData } from "@flare/tuitio-client";

const API_URL = env.VITE_APP_API_URL;
const getHeaders = (): Record<string, string> => {
  const { token } = fetchTuitioData();

  const headers: Record<string, string> = {
    "Content-Type": "application/json"
  };

  if (token) {
    headers.Authorization = `Tuitio ${token}`;
  }

  return headers;
};

const headers = getHeaders();
const axiosInstance = axios.create({
  baseURL: API_URL,
  headers
});

const NotificationApiTest: React.FC = () => {
  const { connectionId } = useRealtimeNotifications();
  const [title, setTitle] = useState("");
  const [message, setMessage] = useState("");
  const [userId, setUserId] = useState("");
  const [responseStatus, setResponseStatus] = useState<{ success: boolean; message: string } | null>(null);

  // Handle broadcasting a notification through the API
  const handleBroadcastNotification = async () => {
    try {
      const response = await axiosInstance.post("/api/realtime-notifications/broadcast", {
        title: title || "API Test Notification",
        message: message || `This is a test notification sent from the API at ${new Date().toLocaleTimeString()}`
      });

      console.log("Broadcast response:", response.data);
      setResponseStatus({ success: true, message: "Broadcast sent successfully" });
      setTitle("");
      setMessage("");
    } catch (error) {
      console.error("Error sending broadcast:", error);
      setResponseStatus({ success: false, message: "Failed to send broadcast" });
    }
  };

  // Handle sending a notification to a specific user
  const handleSendToUser = async () => {
    try {
      const response = await axiosInstance.post("/api/Notifications/user", {
        userId: userId,
        title: title || "User-specific Notification",
        message: message || `This notification was sent to user ${userId} at ${new Date().toLocaleTimeString()}`
      });

      console.log("User notification response:", response.data);
      setResponseStatus({ success: true, message: "User notification sent successfully" });
      setTitle("");
      setMessage("");
    } catch (error) {
      console.error("Error sending user notification:", error);
      setResponseStatus({ success: false, message: "Failed to send user notification" });
    }
  };

  return (
    <Box sx={{ p: 1 }}>
      <Typography variant="h6" gutterBottom>
        Test Notification API Endpoints
      </Typography>

      {/* Display connection ID info */}
      <Box sx={{ mb: 2 }}>
        <Typography variant="body2" color="textSecondary">
          {connectionId ? `Current connection ID: ${connectionId}` : "Not connected to SignalR hub. Connect first to receive notifications."}
        </Typography>
      </Box>

      <Grid container spacing={2}>
        <Grid size={{ xs: 12, md: 6 }}>
          <Card>
            <CardContent>
              <Typography variant="h6" gutterBottom>
                Broadcast Notification (API)
              </Typography>
              <TextField label="Title" fullWidth value={title} onChange={e => setTitle(e.target.value)} margin="normal" />
              <TextField label="Message" fullWidth value={message} onChange={e => setMessage(e.target.value)} margin="normal" />
              <Button variant="contained" color="primary" startIcon={<SendIcon />} onClick={handleBroadcastNotification} sx={{ mt: 2 }}>
                Broadcast via API
              </Button>
            </CardContent>
          </Card>
        </Grid>

        <Grid size={{ xs: 12, md: 6 }}>
          <Card>
            <CardContent>
              <Typography variant="h6" gutterBottom>
                User-specific Notification (API)
              </Typography>
              <TextField
                label="User ID"
                fullWidth
                value={userId}
                onChange={e => setUserId(e.target.value)}
                placeholder="Enter connection ID to receive notification"
                margin="normal"
              />
              <TextField label="Title" fullWidth value={title} onChange={e => setTitle(e.target.value)} margin="normal" />
              <TextField label="Message" fullWidth value={message} onChange={e => setMessage(e.target.value)} margin="normal" />
              <Button variant="contained" color="warning" startIcon={<SendIcon />} onClick={handleSendToUser} sx={{ mt: 2 }} disabled={!userId.trim()}>
                Send to User via API
              </Button>
            </CardContent>
          </Card>
        </Grid>
      </Grid>

      {/* Response status */}
      {responseStatus && (
        <Box
          sx={{
            mt: 2,
            p: 2,
            borderRadius: 1,
            bgcolor: responseStatus.success ? "success.light" : "error.light"
          }}
        >
          <Typography variant="body2" color="textPrimary">
            {responseStatus.message}
          </Typography>
        </Box>
      )}

      <Box sx={{ mt: 2 }}>
        <Typography variant="body2" color="textSecondary">
          Note: For user-specific notifications, enter a valid connection ID in the User ID field. You can find your connection ID at the top of this page when
          connected to SignalR.
        </Typography>
      </Box>
    </Box>
  );
};

export default NotificationApiTest;
