import React, { useState, useEffect } from "react";
import {
  Alert,
  Badge,
  Box,
  Button,
  Drawer,
  IconButton,
  List,
  ListItem,
  Snackbar,
  Typography,
  Paper,
  TextField,
  Grid,
  Card,
  CardContent,
  Chip,
  Divider
} from "@mui/material";
import NotificationsIcon from "@mui/icons-material/Notifications";
import CloseIcon from "@mui/icons-material/Close";
import SendIcon from "@mui/icons-material/Send";
import DeleteIcon from "@mui/icons-material/Delete";
import { useRealtimeNotifications } from "units/notifications/hooks";
import { ConnectionStatus } from "./parts";
import NotificationApiTest from "./NotificationApiTest";

type TestNotification = {
  title: string;
  message: string;
};

const TEST_NOTIFICATION_TYPE = "Notifications.Test";

const NotificationDemo: React.FC = () => {
  const { notifications, messages, sendMessage, sendNotification, joinGroup, sendToGroup, sendToClient, sendToUser } = useRealtimeNotifications();
  const [open, setOpen] = useState(false);
  const [openSnackbar, setOpenSnackbar] = useState(false);
  const [currentNotification, setCurrentNotification] = useState<TestNotification>({ title: "", message: "" });

  // Form states
  const [messageText, setMessageText] = useState("");
  const [notificationTitle, setNotificationTitle] = useState("");
  const [notificationMessage, setNotificationMessage] = useState("");
  const [groupName, setGroupName] = useState("");
  const [groupMessage, setGroupMessage] = useState("");
  const [joinedGroups, setJoinedGroups] = useState<string[]>([]);
  const [userId, setUserId] = useState("");
  const [clientConnectionId, setClientConnectionId] = useState("");
  const [userNotificationTitle, setUserNotificationTitle] = useState("");
  const [userNotificationMessage, setUserNotificationMessage] = useState("");

  // Handle sending a test message
  const handleSendTestMessage = () => {
    if (messageText.trim()) {
      sendMessage(messageText);
      setMessageText("");
    } else {
      sendMessage(`Test message at ${new Date().toLocaleTimeString()}`);
    }
  };

  // Handle sending a test notification
  const handleSendTestNotification = () => {
    if (notificationTitle.trim() && notificationMessage.trim()) {
      const payload: TestNotification = {
        title: notificationTitle,
        message: notificationMessage
      };
      const notification = {
        type: TEST_NOTIFICATION_TYPE,
        payload
      };
      sendNotification(notification);
      setNotificationTitle("");
      setNotificationMessage("");
    } else {
      const payload: TestNotification = {
        title: "Test Notification",
        message: `This is a test notification sent at ${new Date().toLocaleTimeString()}`
      };

      const notification = {
        type: TEST_NOTIFICATION_TYPE,
        payload
      };

      sendNotification(notification);
    }
  };

  // Handle joining a group
  const handleJoinGroup = () => {
    if (groupName.trim() && !joinedGroups.includes(groupName)) {
      joinGroup(groupName);
      setJoinedGroups(prev => [...prev, groupName]);
    }
  };

  // Handle sending a message to a group
  const handleSendToGroup = () => {
    if (groupName.trim() && groupMessage.trim()) {
      sendToGroup(groupName, groupMessage);
      setGroupMessage("");
    }
  };

  // Handle sending a notification to a specific user
  const handleSendToUser = () => {
    if (userId.trim() && userNotificationTitle.trim() && userNotificationMessage.trim()) {
      sendToUser(userId, userNotificationTitle, userNotificationMessage);
      setUserNotificationTitle("");
      setUserNotificationMessage("");
    }
  };

  const handleSendToClient = () => {
    if (!clientConnectionId.trim() || !messageText.trim()) return;
    sendToClient(clientConnectionId, messageText);
    setClientConnectionId("");
    setMessageText("");
  };

  // Clear all messages
  const handleClearMessages = () => {
    // We can't directly clear the messages in the SignalR context,
    // but we can simulate a refresh
    window.location.reload();
  };

  // Show the most recent notification in a snackbar
  useEffect(() => {
    if (notifications.length > 0) {
      const latestNotification = notifications[notifications.length - 1];
      let payload: TestNotification;

      if (latestNotification.type !== TEST_NOTIFICATION_TYPE) {
        payload = {
          title: "Unknown Notification",
          message: `Received notification of type ${latestNotification.type}`
        };
      } else {
        payload = latestNotification.payload as TestNotification;
      }

      setCurrentNotification(payload);
      setOpenSnackbar(true);
    }
  }, [notifications]);

  return (
    <>
      <Box sx={{ p: 1 }}>
        <Box sx={{ mb: 2, display: "flex", alignItems: "center", justifyContent: "space-between" }}>
          <Typography variant="h6">SignalR Demo</Typography>
          <Box sx={{ display: "flex", alignItems: "center" }}>
            <IconButton color="inherit" onClick={() => setOpen(true)}>
              <Badge badgeContent={notifications.length} color="error">
                <NotificationsIcon />
              </Badge>
            </IconButton>
            <ConnectionStatus />
          </Box>
        </Box>

        <Grid container spacing={2}>
          <Grid size={{ xs: 12, md: 6 }}>
            <Card>
              <CardContent>
                <Typography variant="h6" gutterBottom>
                  Send message
                </Typography>
                <TextField label="Message" fullWidth value={messageText} onChange={e => setMessageText(e.target.value)} margin="normal" />
                <Button variant="contained" startIcon={<SendIcon />} onClick={handleSendTestMessage} sx={{ mt: 2 }}>
                  Send test message
                </Button>
              </CardContent>
            </Card>
          </Grid>

          <Grid size={{ xs: 12, md: 6 }}>
            <Card>
              <CardContent>
                <Typography variant="h6" gutterBottom>
                  Send notification
                </Typography>
                <TextField label="Title" fullWidth value={notificationTitle} onChange={e => setNotificationTitle(e.target.value)} margin="normal" />
                <TextField label="Message" fullWidth value={notificationMessage} onChange={e => setNotificationMessage(e.target.value)} margin="normal" />
                <Button variant="contained" color="secondary" startIcon={<SendIcon />} onClick={handleSendTestNotification} sx={{ mt: 2 }}>
                  Send notification
                </Button>
              </CardContent>
            </Card>
          </Grid>

          <Grid size={{ xs: 12, md: 6 }}>
            <Card>
              <CardContent>
                <Typography variant="h6" gutterBottom>
                  Group messages
                </Typography>
                <TextField label="Group name" fullWidth value={groupName} onChange={e => setGroupName(e.target.value)} margin="normal" />
                <Button variant="outlined" onClick={handleJoinGroup} sx={{ mt: 2, mr: 2 }}>
                  Join group
                </Button>

                {joinedGroups.length > 0 && (
                  <Box sx={{ mt: 2 }}>
                    <Typography variant="subtitle2" gutterBottom>
                      Joined groups:
                    </Typography>
                    <Box sx={{ display: "flex", flexWrap: "wrap", gap: 1 }}>
                      {joinedGroups.map((group, index) => (
                        <Chip key={index} label={group} />
                      ))}
                    </Box>
                  </Box>
                )}

                <TextField label="Group message" fullWidth value={groupMessage} onChange={e => setGroupMessage(e.target.value)} margin="normal" />
                <Button
                  variant="contained"
                  color="primary"
                  startIcon={<SendIcon />}
                  onClick={handleSendToGroup}
                  sx={{ mt: 2 }}
                  disabled={joinedGroups.length === 0}
                >
                  Send to group
                </Button>
              </CardContent>
            </Card>
          </Grid>

          <Grid size={{ xs: 12, md: 6 }}>
            <Card>
              <CardContent>
                <Typography variant="h6" gutterBottom>
                  Client-specific message
                </Typography>
                <TextField label="Connection ID" fullWidth value={clientConnectionId} onChange={e => setClientConnectionId(e.target.value)} margin="normal" />
                <TextField label="Message" fullWidth value={messageText} onChange={e => setMessageText(e.target.value)} margin="normal" />
                <Button
                  variant="contained"
                  color="warning"
                  startIcon={<SendIcon />}
                  onClick={handleSendToClient}
                  sx={{ mt: 2 }}
                  disabled={!clientConnectionId.trim()}
                >
                  Send to client
                </Button>
              </CardContent>
            </Card>
          </Grid>

          <Grid size={{ xs: 12, md: 6 }}>
            <Card>
              <CardContent>
                <Typography variant="h6" gutterBottom>
                  User-specific notifications
                </Typography>
                <TextField label="User ID" fullWidth value={userId} onChange={e => setUserId(e.target.value)} margin="normal" />
                <TextField label="Title" fullWidth value={userNotificationTitle} onChange={e => setUserNotificationTitle(e.target.value)} margin="normal" />
                <TextField
                  label="Message"
                  fullWidth
                  value={userNotificationMessage}
                  onChange={e => setUserNotificationMessage(e.target.value)}
                  margin="normal"
                />
                <Button variant="contained" color="warning" startIcon={<SendIcon />} onClick={handleSendToUser} sx={{ mt: 2 }} disabled={!userId.trim()}>
                  Send to user
                </Button>
              </CardContent>
            </Card>
          </Grid>

          <Grid size={{ xs: 12, md: 6 }}>
            <Card>
              <CardContent>
                <Typography variant="h6" gutterBottom>
                  Received messages
                </Typography>
                <Box sx={{ maxHeight: "200px", overflowY: "auto", border: 1, borderColor: "divider", p: 1 }}>
                  {messages.length === 0 ? (
                    <Typography variant="body2" color="text.secondary">
                      No messages received yet
                    </Typography>
                  ) : (
                    <List dense>
                      {messages.map((message, index) => (
                        <ListItem key={index} divider={index < messages.length - 1}>
                          <Typography variant="body2">{message}</Typography>
                        </ListItem>
                      ))}
                    </List>
                  )}
                </Box>
                <Button variant="outlined" color="error" startIcon={<DeleteIcon />} onClick={handleClearMessages} sx={{ mt: 2 }}>
                  Clear messages
                </Button>
              </CardContent>
            </Card>
          </Grid>
        </Grid>

        {/* Notification Drawer */}
        <Drawer anchor="right" open={open} onClose={() => setOpen(false)}>
          <Box sx={{ width: 320, p: 2 }}>
            <Box sx={{ display: "flex", justifyContent: "space-between", alignItems: "center", mb: 2 }}>
              <Typography variant="h6">Notifications</Typography>
              <IconButton onClick={() => setOpen(false)}>
                <CloseIcon />
              </IconButton>
            </Box>

            {notifications.length === 0 ? (
              <Paper elevation={0} sx={{ p: 2, textAlign: "center" }}>
                <Typography variant="body2" color="text.secondary">
                  No notifications
                </Typography>
              </Paper>
            ) : (
              <List>
                {notifications.map((notification, index) => (
                  <Paper key={index} elevation={1} sx={{ mb: 2, overflow: "hidden" }}>
                    <ListItem sx={{ bgcolor: "background.paper" }}>
                      <Box sx={{ width: "100%" }}>
                        <Typography variant="subtitle1" sx={{ fontWeight: "bold" }}>
                          {notification.type}
                        </Typography>
                        <Typography variant="body2">{JSON.stringify(notification.payload)}</Typography>
                      </Box>
                    </ListItem>
                  </Paper>
                ))}
              </List>
            )}
          </Box>
        </Drawer>

        {/* Notification Snackbar */}
        <Snackbar open={openSnackbar} autoHideDuration={6000} onClose={() => setOpenSnackbar(false)} anchorOrigin={{ vertical: "bottom", horizontal: "right" }}>
          <Alert onClose={() => setOpenSnackbar(false)} severity="info" sx={{ width: "100%" }}>
            <Typography variant="subtitle2">{currentNotification.title}</Typography>
            <Typography variant="body2">{currentNotification.message}</Typography>
          </Alert>
        </Snackbar>
      </Box>
      <Divider sx={{ my: 2 }} />
      <NotificationApiTest />
    </>
  );
};

export default NotificationDemo;
