import React, { useEffect, useState, ReactNode, useCallback, useRef } from "react";
import signalRService from "./signalRService";
import ErrorBox from "./components/ErrorBox";
import ConnectingBox from "./components/ConnectingBox";
import { MAX_RETRY_ATTEMPTS, RETRY_DELAY, NOTIFICATION_HISTORY_LIMIT } from "./constants";
import { RealtimeNotification, RealtimeNotificationListeners, SubscribeFn, PublishFn } from "./types";
import RealtimeNotificationsContext, { RealtimeNotificationsContextData } from "./RealtimeNotificationsContext";

interface SignalRProviderProps {
  children: ReactNode;
}

const RealtimeNotificationsProvider: React.FC<SignalRProviderProps> = ({ children }) => {
  const [isConnected, setIsConnected] = useState(false);
  const [isConnecting, setIsConnecting] = useState(true);
  const [connectionError, setConnectionError] = useState<string | null>(null);
  const [messages, setMessages] = useState<string[]>([]);
  const [notifications, setNotifications] = useState<RealtimeNotification[]>([]);
  const [connectionAttempts, setConnectionAttempts] = useState(0);

  const listenersRef = useRef<RealtimeNotificationListeners>({});
  const queuedNotificationsRef = useRef<RealtimeNotification[]>([]);

  const safePublish: PublishFn = useCallback(
    <T,>(event: RealtimeNotification<T>) => {
      if (!isConnected) return;
      signalRService.sendNotification(event);
    },
    [isConnected]
  );

  const publishQueuedNotifications = useCallback(() => {
    queuedNotificationsRef.current.forEach(event => {
      safePublish(event);
    });
    queuedNotificationsRef.current = [];
  }, [safePublish]);

  // a ref is used here to keep publishQueuedNotifications outside of onReconnected dependencies.
  const publishQueuedNotificationsRef = useRef(publishQueuedNotifications);
  publishQueuedNotificationsRef.current = publishQueuedNotifications;

  useEffect(() => {
    const retryLimitReached = connectionAttempts >= MAX_RETRY_ATTEMPTS;
    if (retryLimitReached) return;

    const connectToSignalR = () => {
      // Start SignalR connection
      signalRService.startConnection({
        onStart: () => {
          setIsConnecting(true);
          setConnectionError(null);
        },
        onSuccess: () => {
          setIsConnected(true);
          setConnectionAttempts(0);

          // Set up listeners
          signalRService.addMessageListener(message => {
            setMessages(prev => [...prev, message]);
          });

          signalRService.addNotificationListener(notification => {
            setNotifications(prev => {
              const notifications = [...prev, notification];
              // Limit notification history to avoid memory issues
              if (notifications.length > NOTIFICATION_HISTORY_LIMIT) {
                notifications.shift();
              }
              return notifications;
            });

            // Call any registered listeners for this notification type
            if (listenersRef.current[notification.type]) {
              listenersRef.current[notification.type].forEach(callback => callback(notification));
            }
          });

          signalRService.addGroupMessageListener((group, message) => {
            setMessages(prev => [...prev, `Group ${group}: ${message}`]);
          });
        },
        onError: error => {
          const newAttempts = connectionAttempts + 1;
          if (newAttempts >= MAX_RETRY_ATTEMPTS) {
            const msg = "Failed to connect to SignalR hub after multiple attempts.";
            setConnectionError(error instanceof Error ? `${msg} ${error.message}` : msg);
            setIsConnecting(false);
          } else {
            setConnectionError(`Retrying connection (Attempt ${newAttempts} of ${MAX_RETRY_ATTEMPTS})...`);
          }
          setConnectionAttempts(newAttempts);
        },
        onEnd: () => {
          setIsConnecting(false);
        }
      });
    };

    const isRetry = connectionAttempts > 0;
    if (isRetry) {
      const timer = setTimeout(connectToSignalR, RETRY_DELAY);
      return () => {
        clearTimeout(timer);
        signalRService.stopConnection();
      };
    } else {
      connectToSignalR();
    }

    // Check connection state changes
    signalRService.connection.onreconnecting(() => {
      setIsConnected(false);
      setIsConnecting(true);
    });

    signalRService.connection.onreconnected(() => {
      setIsConnected(true);
      setIsConnecting(false);
      publishQueuedNotificationsRef.current();
    });

    signalRService.connection.onclose(() => {
      setIsConnected(false);
      setConnectionError("Connection closed unexpectedly");
    });

    // Clean up on unmount
    return () => {
      signalRService.stopConnection();
    };
  }, [connectionAttempts]);

  const subscribe: SubscribeFn = useCallback(<T,>(notificationType: string, callback: (notification: RealtimeNotification<T>) => void) => {
    if (!listenersRef.current[notificationType]) {
      listenersRef.current[notificationType] = [];
    }
    listenersRef.current[notificationType].push(callback);

    const unsubscribe = () => {
      listenersRef.current[notificationType] = listenersRef.current[notificationType].filter(cb => cb !== callback);
    };
    return unsubscribe;
  }, []);

  const publish: PublishFn = useCallback(
    <T,>(event: RealtimeNotification<T>) => {
      if (!signalRService.connection) return;
      if (signalRService.isConnecting()) {
        // queue notifications while connecting/reconnecting
        queuedNotificationsRef.current.push(event);
        return;
      }

      if (isConnected) {
        return;
      }

      safePublish(event);
    },
    [isConnected, safePublish]
  );

  const value: RealtimeNotificationsContextData = {
    connectionId: signalRService.connection.connectionId,
    isConnected,
    notifications,
    subscribe,
    publish,
    messages,
    sendMessage: signalRService.sendMessage,
    sendNotification: signalRService.sendNotification,
    joinGroup: signalRService.joinGroup,
    sendToGroup: signalRService.sendToGroup,
    sendToClient: signalRService.sendToClient,
    sendToUser: signalRService.sendToUser
  };

  const handleRetryConnection = useCallback(() => {
    setConnectionAttempts(0);
    setConnectionError(null);
  }, []);

  const handleAgreeAndContinue = useCallback(() => {
    setConnectionError(null);
  }, []);

  // Show loading indicator while connecting
  if (isConnecting) {
    return <ConnectingBox connectionAttempts={connectionAttempts} />;
  }

  // Show error if connection failed
  if (connectionError) {
    const retryLimitReached = connectionAttempts >= MAX_RETRY_ATTEMPTS;
    return <ErrorBox message={connectionError} showActions={retryLimitReached} onRetry={handleRetryConnection} onAgreeAndContinue={handleAgreeAndContinue} />;
  }

  // Render children only when connected or user has chosen to continue without connection.
  return <RealtimeNotificationsContext.Provider value={value}>{children}</RealtimeNotificationsContext.Provider>;
};

export default RealtimeNotificationsProvider;
