import * as signalR from "@microsoft/signalr";
import env from "utils/env";
import { acquire as fetchTuitioData } from "@flare/tuitio-client";
import type { ConnectionStartOptions, RealtimeNotification } from "./types";

const API_URL = env.VITE_APP_API_URL;

// Create a SignalR connection
const connection = new signalR.HubConnectionBuilder()
  .withUrl(`${API_URL}/hubs/notifications`, {
    skipNegotiation: false,
    transport: signalR.HttpTransportType.WebSockets,
    accessTokenFactory: () => {
      const { token } = fetchTuitioData();
      if (!token) {
        console.error("No token found for SignalR connection");
        return "";
      }
      return token;
    }
  })
  .withAutomaticReconnect() // Automatically reconnect if connection is lost
  .configureLogging(signalR.LogLevel.Information)
  .build();

// Start the connection
const startConnection = async (options?: ConnectionStartOptions) => {
  try {
    options?.onStart?.();
    await connection.start();
    const connected = connection.state === signalR.HubConnectionState.Connected;
    if (!connected) {
      throw new Error("Connection did not reach connected state");
    }
    console.log("SignalR Connected");
    options?.onSuccess?.();
  } catch (err) {
    console.error("SignalR Connection Error: ", err);
    options?.onError?.(err);
    // Retry connection after a delay
    //setTimeout(startConnection, 5000);
  } finally {
    options?.onEnd?.();
  }
};

const stopConnection = () => {
  connection.off("ReceiveMessage");
  connection.off("ReceiveNotification");
  connection.off("ReceiveGroupMessage");
  connection.stop();
};

const isConnecting = () => {
  return [signalR.HubConnectionState.Connecting, signalR.HubConnectionState.Reconnecting].includes(connection.state);
};

// Add event handlers for common SignalR events
const addMessageListener = (callback: (message: string) => void) => {
  connection.on("ReceiveMessage", message => {
    callback(message);
  });
};

const addNotificationListener = (callback: (notification: RealtimeNotification) => void) => {
  connection.on("ReceiveNotification", notification => {
    callback(notification);
  });
};

const addGroupMessageListener = (callback: (group: string, message: string) => void) => {
  connection.on("ReceiveGroupMessage", (group, message) => {
    callback(group, message);
  });
};

// Methods to call SignalR hub methods
const sendMessage = async (message: string) => {
  try {
    await connection.invoke("SendMessage", message);
  } catch (err) {
    console.error("Error sending message: ", err);
  }
};

const sendNotification = async (notification: RealtimeNotification) => {
  try {
    await connection.invoke("SendNotification", notification);
  } catch (err) {
    console.error("Error sending notification: ", err);
  }
};

const joinGroup = async (groupName: string) => {
  try {
    await connection.invoke("JoinGroup", groupName);
  } catch (err) {
    console.error("Error joining group: ", err);
  }
};

const sendToGroup = async (groupName: string, message: string) => {
  try {
    await connection.invoke("SendToGroup", groupName, message);
  } catch (err) {
    console.error("Error sending to group: ", err);
  }
};

const sendToClient = async (connectionId: string, message: string) => {
  try {
    await connection.invoke("SendToClient", connectionId, message);
  } catch (err) {
    console.error("Error sending to client: ", err);
  }
};

const sendToUser = async (userId: string, title: string, message: string) => {
  try {
    await connection.invoke("SendToUser", userId, title, message);
  } catch (err) {
    console.error("Error sending notification to user: ", err);
  }
};

// Export the connection and methods
export default {
  connection,
  startConnection,
  stopConnection,
  isConnecting,
  addMessageListener,
  addNotificationListener,
  addGroupMessageListener,
  sendMessage,
  sendNotification,
  joinGroup,
  sendToGroup,
  sendToClient,
  sendToUser
};
